{
    "$graph": [
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "label": "Input BAM file",
                    "doc": "Input BAM file, sorted by coordinates",
                    "id": "#bam-bedgraph-bigwig.cwl/bam_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "label": "bedGraph output filename",
                    "doc": "Output filename for generated bedGraph",
                    "id": "#bam-bedgraph-bigwig.cwl/bedgraph_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "label": "bigWig output filename",
                    "doc": "Output filename for generated bigWig",
                    "id": "#bam-bedgraph-bigwig.cwl/bigwig_filename"
                },
                {
                    "type": "File",
                    "label": "Chromosome length file",
                    "doc": "Tab delimited chromosome length file: <chromName><TAB><chromSize>",
                    "id": "#bam-bedgraph-bigwig.cwl/chrom_length_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "label": "Enable dUTP",
                    "doc": "Change strand af the mate read, so both reads come from the same strand",
                    "id": "#bam-bedgraph-bigwig.cwl/dutp"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "label": "Fixed fragment size",
                    "doc": "Set fixed fragment size for genome coverage calculation",
                    "id": "#bam-bedgraph-bigwig.cwl/fragment_size"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "label": "Mapped reads number",
                    "doc": "Parameter to calculate scale as 1000000/mapped_reads_number. Ignored by bedtools-genomecov.cwl in\nbam_to_bedgraph step if scale is provided\n",
                    "id": "#bam-bedgraph-bigwig.cwl/mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "label": "Enable paired-end genome coverage calculation",
                    "doc": "Enable paired-end genome coverage calculation",
                    "id": "#bam-bedgraph-bigwig.cwl/pairchip"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "label": "Genome coverage scaling coefficient",
                    "doc": "Coefficient to scale the genome coverage by a constant factor",
                    "id": "#bam-bedgraph-bigwig.cwl/scale"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "label": "Split reads by 'N' and 'D'",
                    "doc": "Calculate genome coverage for each part of the splitted by 'N' and 'D' read",
                    "id": "#bam-bedgraph-bigwig.cwl/split"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "label": "Enable strand specific genome coverage calculation",
                    "doc": "Calculate genome coverage of intervals from a specific strand",
                    "id": "#bam-bedgraph-bigwig.cwl/strand"
                }
            ],
            "steps": [
                {
                    "run": "#bedtools-genomecov.cwl",
                    "in": [
                        {
                            "default": "-bg",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/depth"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/dutp",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/du"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/fragment_size",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/fragment_size"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bam_file",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/input_file"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/mapped_reads_number",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/mapped_reads_number"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bedgraph_filename",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/output_filename"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/pairchip",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/pairchip"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/scale",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/scale"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/split",
                            "valueFrom": "${\n  if (self == null){\n    return true;\n  } else {\n    return self;\n  }\n}\n",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/split"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/strand",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/strand"
                        }
                    ],
                    "out": [
                        "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/genome_coverage_file"
                    ],
                    "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph"
                },
                {
                    "run": "#linux-sort.cwl",
                    "in": [
                        {
                            "default": [
                                "1,1",
                                "2,2n"
                            ],
                            "id": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/key"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/genome_coverage_file",
                            "id": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/unsorted_file"
                        }
                    ],
                    "out": [
                        "#bam-bedgraph-bigwig.cwl/sort_bedgraph/sorted_file"
                    ],
                    "id": "#bam-bedgraph-bigwig.cwl/sort_bedgraph"
                },
                {
                    "run": "#ucsc-bedgraphtobigwig.cwl",
                    "in": [
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/sorted_file",
                            "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/bedgraph_file"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/chrom_length_file",
                            "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/chrom_length_file"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bigwig_filename",
                            "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/output_filename"
                        }
                    ],
                    "out": [
                        "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/bigwig_file"
                    ],
                    "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig"
                }
            ],
            "doc": "Workflow converts input BAM file into bigWig and bedGraph files\n",
            "id": "#bam-bedgraph-bigwig.cwl",
            "http://schema.org/name": "bam-bedgraph-bigwig",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/workflows/bam-bedgraph-bigwig.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "id": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "id": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Workflow converts input BAM file into bigWig and bedGraph files.\n\nInput BAM file should be sorted by coordinates (required by `bam_to_bedgraph` step).\n\nIf `split` input is not provided use true by default. Default logic is implemented in `valueFrom` field of `split`\ninput inside `bam_to_bedgraph` step to avoid possible bug in cwltool with setting default values for workflow inputs.\n\n`scale` has higher priority over the `mapped_reads_number`. The last one is used to calculate `-scale` parameter for\n`bedtools genomecov` (step `bam_to_bedgraph`) only in a case when input `scale` is not provided. All logic is\nimplemented inside `bedtools-genomecov.cwl`.\n\n`bigwig_filename` defines the output name only for generated bigWig file. `bedgraph_filename` defines the output name\nfor generated bedGraph file and can influence on generated bigWig filename in case when `bigwig_filename` is not provided.\n\nAll workflow inputs and outputs don't have `format` field to avoid format incompatibility errors when workflow is used\nas subworkflow.",
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/sorted_file",
                    "label": "bedGraph output file",
                    "doc": "bedGraph output file",
                    "id": "#bam-bedgraph-bigwig.cwl/bedgraph_file"
                },
                {
                    "type": "File",
                    "outputSource": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/bigwig_file",
                    "label": "bigWig output file",
                    "doc": "bigWig output file",
                    "id": "#bam-bedgraph-bigwig.cwl/bigwig_file"
                }
            ]
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "SubworkflowFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "Directory",
                    "id": "#main/bowtie2_indices_folder"
                },
                {
                    "type": "File",
                    "id": "#main/chr_length_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "id": "#main/downstream_fastq"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "id": "#main/paired"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "id": "#main/threads"
                },
                {
                    "type": "File",
                    "id": "#main/upstream_fastq"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#main/samtools_sort_index_after_dup_removing/bam_bai_pair",
                    "id": "#main/bam_file"
                },
                {
                    "type": "File",
                    "outputSource": "#main/bamtools_stats/log_file",
                    "id": "#main/bamtools_log"
                },
                {
                    "type": "File",
                    "outputSource": "#main/bam_to_bigwig/bedgraph_file",
                    "id": "#main/bed"
                },
                {
                    "type": "File",
                    "outputSource": "#main/bam_to_bigwig/bigwig_file",
                    "id": "#main/bigwig"
                },
                {
                    "type": "File",
                    "outputSource": "#main/bowtie2_aligner/output_log",
                    "id": "#main/bowtie2_log"
                },
                {
                    "type": "File",
                    "outputSource": "#main/remove_dup_picard/metrics_file",
                    "id": "#main/picard_metrics"
                }
            ],
            "steps": [
                {
                    "run": "#bam-bedgraph-bigwig.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_sort_index_after_dup_removing/bam_bai_pair",
                            "id": "#main/bam_to_bigwig/bam_file"
                        },
                        {
                            "source": "#main/chr_length_file",
                            "id": "#main/bam_to_bigwig/chrom_length_file"
                        },
                        {
                            "source": "#main/bamtools_stats/mapped_reads_number",
                            "id": "#main/bam_to_bigwig/mapped_reads_number"
                        },
                        {
                            "source": "#main/paired",
                            "id": "#main/bam_to_bigwig/pairchip"
                        }
                    ],
                    "out": [
                        "#main/bam_to_bigwig/bigwig_file",
                        "#main/bam_to_bigwig/bedgraph_file"
                    ],
                    "id": "#main/bam_to_bigwig"
                },
                {
                    "run": "#bamtools-stats.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_sort_index_after_dup_removing/bam_bai_pair",
                            "id": "#main/bamtools_stats/bam_file"
                        }
                    ],
                    "out": [
                        "#main/bamtools_stats/mapped_reads_number",
                        "#main/bamtools_stats/log_file"
                    ],
                    "id": "#main/bamtools_stats"
                },
                {
                    "run": "#bowtie2.cwl",
                    "in": [
                        {
                            "source": "#main/upstream_fastq",
                            "id": "#main/bowtie2_aligner/filelist"
                        },
                        {
                            "source": "#main/downstream_fastq",
                            "id": "#main/bowtie2_aligner/filelist_mates"
                        },
                        {
                            "source": "#main/bowtie2_indices_folder",
                            "id": "#main/bowtie2_aligner/indices_folder"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/bowtie2_aligner/threads"
                        }
                    ],
                    "out": [
                        "#main/bowtie2_aligner/output",
                        "#main/bowtie2_aligner/output_log"
                    ],
                    "id": "#main/bowtie2_aligner"
                },
                {
                    "run": "#picard-markduplicates.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_sort_index/bam_bai_pair",
                            "id": "#main/remove_dup_picard/input_file"
                        },
                        {
                            "default": true,
                            "id": "#main/remove_dup_picard/remove_dup"
                        }
                    ],
                    "out": [
                        "#main/remove_dup_picard/output_file",
                        "#main/remove_dup_picard/metrics_file"
                    ],
                    "id": "#main/remove_dup_picard"
                },
                {
                    "run": "#samtools-sort-index.cwl",
                    "in": [
                        {
                            "source": "#main/bowtie2_aligner/output",
                            "id": "#main/samtools_sort_index/sort_input"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/samtools_sort_index/threads"
                        }
                    ],
                    "out": [
                        "#main/samtools_sort_index/bam_bai_pair"
                    ],
                    "id": "#main/samtools_sort_index"
                },
                {
                    "run": "#samtools-sort-index.cwl",
                    "in": [
                        {
                            "source": "#main/remove_dup_picard/output_file",
                            "id": "#main/samtools_sort_index_after_dup_removing/sort_input"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/samtools_sort_index_after_dup_removing/threads"
                        }
                    ],
                    "out": [
                        "#main/samtools_sort_index_after_dup_removing/bam_bai_pair"
                    ],
                    "id": "#main/samtools_sort_index_after_dup_removing"
                }
            ],
            "id": "#main"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ShellCommandRequirement"
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/bamtools:v2.4.1"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "File"
                    ],
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-in"
                    },
                    "doc": "the input BAM file\n",
                    "id": "#bamtools-stats.cwl/bam_file"
                }
            ],
            "outputs": [
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "stats.log",
                        "outputEval": "${\n  var s = self[0].contents.replace(/ /g,'').replace(/ *\\([^)]*\\) */g,'');\n  var duplicates = parseInt(s.substring ( s.indexOf(\"Duplicates\")+11, s.indexOf(\"\\t\", (s.indexOf(\"Duplicates\")))  ));\n  return duplicates;\n}\n"
                    },
                    "id": "#bamtools-stats.cwl/duplicate_reads_number"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "stats.log",
                        "outputEval": "${\n  var s = self[0].contents.replace(/ /g,'').replace(/ *\\([^)]*\\) */g,'');\n  var failedQC = parseInt(s.substring ( s.indexOf(\"FailedQC\")+9, s.indexOf(\"\\t\", (s.indexOf(\"FailedQC\")))  ));\n  return failedQC;\n}\n"
                    },
                    "id": "#bamtools-stats.cwl/failed_QC_reads_number"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "stats.log",
                        "outputEval": "${\n  var s = self[0].contents.replace(/ /g,'').replace(/ *\\([^)]*\\) */g,'');\n  var forwardstrand = parseInt(s.substring ( s.indexOf(\"Forwardstrand\")+14, s.indexOf(\"\\t\", (s.indexOf(\"Forwardstrand\")))  ));\n  return forwardstrand;\n}\n"
                    },
                    "id": "#bamtools-stats.cwl/forward_strand_reads_number"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "stats.log"
                    },
                    "id": "#bamtools-stats.cwl/log_file"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "stats.log",
                        "outputEval": "${\n  var s = self[0].contents.replace(/ /g,'').replace(/ *\\([^)]*\\) */g,'');\n  var mappedreads = parseInt(s.substring ( s.indexOf(\"Mappedreads\")+12, s.indexOf(\"\\t\", (s.indexOf(\"Mappedreads\")))  ));\n  return mappedreads;\n}\n"
                    },
                    "id": "#bamtools-stats.cwl/mapped_reads_number"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "stats.log",
                        "outputEval": "${\n  var s = self[0].contents.replace(/ /g,'').replace(/ *\\([^)]*\\) */g,'');\n  var pairedendreads = parseInt(s.substring ( s.indexOf(\"Paired-endreads\")+16, s.indexOf(\"\\t\", (s.indexOf(\"Paired-endreads\")))  ));\n  return pairedendreads;\n}\n"
                    },
                    "id": "#bamtools-stats.cwl/pairedend_reads_number"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "stats.log",
                        "outputEval": "${\n  var s = self[0].contents.replace(/ /g,'').replace(/ *\\([^)]*\\) */g,'');\n  var reversestrand = parseInt(s.substring ( s.indexOf(\"Reversestrand\")+14, s.indexOf(\"\\t\", (s.indexOf(\"Reversestrand\")))  ));\n  return reversestrand;\n}\n"
                    },
                    "id": "#bamtools-stats.cwl/reverse_strand_reads_number"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "stats.log",
                        "outputEval": "${\n  var s = self[0].contents.replace(/ /g,'').replace(/ *\\([^)]*\\) */g,'');\n  var totalReads = parseInt(s.substring ( s.indexOf(\"Totalreads\")+11, s.indexOf(\"\\t\", (s.indexOf(\"Totalreads\")))  ));\n  return totalReads;\n}\n"
                    },
                    "id": "#bamtools-stats.cwl/total_reads_number"
                }
            ],
            "baseCommand": [
                "bamtools",
                "stats"
            ],
            "arguments": [
                {
                    "valueFrom": "$('> ' + 'stats.log')",
                    "position": 1000,
                    "shellQuote": false
                }
            ],
            "doc": "Tool runs `bamtools stats' to calculate general alignment statistics from the input BAM file\n\n`-insert` parameter is not implemented\n",
            "id": "#bamtools-stats.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#bamtools-metadata.yaml",
                "http://schema.org/name": "bamtools",
                "http://schema.org/about": "Software suite for programmers and end users that facilitates research analysis\nand data management using BAM files. BamTools provides both the first C++ API\npublicly available for BAM file support as well as a command-line toolkit.\n",
                "http://schema.org/url": "https://github.com/pezmaster31/bamtools/wiki",
                "http://schema.org/codeRepository": "https://github.com/pezmaster31/bamtools",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/MIT"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "2.4.1",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#bamtools-metadata.yaml/10.1186/gb-2009-10-3-r25",
                        "datePublished": "14 April 2011",
                        "http://schema.org/name": "BamTools: a C++ API and toolkit for analyzing and managing BAM files",
                        "http://schema.org/author": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Derek W. Barnett"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Erik K. Garrison"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Aaron R. Quinlan"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Michael P. Str\u00f6mberg"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Gabor T. Marth"
                            }
                        ],
                        "http://schema.org/url": "https://academic.oup.com/bioinformatics/article/27/12/1691/255399/BamTools-a-C-API-and-toolkit-for-analyzing-and"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://github.com/pezmaster31/bamtools/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Pacific Biosciences",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Derek Barnett",
                                "http://schema.org/description": "Main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "bamtools-stats",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/bamtools-stats.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:michael.kotliar@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage: bamtools stats [-in <filename> -in <filename> ... | -list <filelist>] [statsOptions]\n\nInput & Output:\n  -in <BAM filename>                the input BAM file [stdin]\n  -list <filename>                  the input BAM file list, one\n                                    line per file\n\nAdditional Stats:\n  -insert                           summarize insert size data\n\nHelp:\n  --help, -h                        shows this help text\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { let ext = (inputs.depth == \"-bg\" || inputs.depth == \"-bga\")?\".bedGraph\":\".tab\"; return inputs.input_file.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.') + ext; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/bedtools2:v2.26.0"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 17,
                        "prefix": "-g"
                    },
                    "doc": "Input genome file. Needed only when -i flag. The genome file is tab delimited <chromName><TAB><chromSize>\n",
                    "id": "#bedtools-genomecov.cwl/chrom_length_file"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "name": "#bedtools-genomecov.cwl/depth/depth",
                            "symbols": [
                                "#bedtools-genomecov.cwl/depth/depth/-bg",
                                "#bedtools-genomecov.cwl/depth/depth/-bga",
                                "#bedtools-genomecov.cwl/depth/depth/-d",
                                "#bedtools-genomecov.cwl/depth/depth/-dz"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "Report the depth type. By default, bedtools genomecov will compute a histogram of coverage\nfor the genome file provided (intputs.chrom_length_file)\n",
                    "id": "#bedtools-genomecov.cwl/depth"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-du"
                    },
                    "doc": "Change strand af the mate read (so both reads from the same strand) useful for strand specific.\nWorks for BAM files only\n",
                    "id": "#bedtools-genomecov.cwl/du"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "-fs"
                    },
                    "doc": "Set fixed fragment size\n",
                    "id": "#bedtools-genomecov.cwl/fragment_size"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 16,
                        "valueFrom": "${\n  var prefix = ((/.*\\.bam$/i).test(inputs.input_file.path))?'-ibam':'-i';\n  return [prefix, inputs.input_file.path];\n}\n"
                    },
                    "doc": "The input file can be in BAM format (Note: BAM must be sorted by position) or <bed/gff/vcf>.\nPrefix is selected on the base of input file extension\n",
                    "id": "#bedtools-genomecov.cwl/input_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 15,
                        "prefix": "-3"
                    },
                    "doc": "Calculate coverage of 3\" positions (instead of entire interval)\n",
                    "id": "#bedtools-genomecov.cwl/m3"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 14,
                        "prefix": "-5"
                    },
                    "doc": "Calculate coverage of 5\" positions (instead of entire interval)\n",
                    "id": "#bedtools-genomecov.cwl/m5"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 7,
                        "prefix": "-scale",
                        "valueFrom": "${\n  if (inputs.scale){\n    return null;\n  } else if (inputs.mapped_reads_number) {\n    return 1000000/inputs.mapped_reads_number;\n  } else {\n    return null;\n  }\n}\n"
                    },
                    "doc": "Optional parameter to calculate scale as 1000000/mapped_reads_number if inputs.scale is not provided\n",
                    "id": "#bedtools-genomecov.cwl/mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "-max"
                    },
                    "doc": "Combine all positions with a depth >= max into\na single bin in the histogram. Irrelevant\nfor -d and -bedGraph\n- (INTEGER)\n",
                    "id": "#bedtools-genomecov.cwl/max"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Name for generated output file\n",
                    "id": "#bedtools-genomecov.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 10,
                        "prefix": "-pc"
                    },
                    "doc": "pair-end chip seq experiment\n",
                    "id": "#bedtools-genomecov.cwl/pairchip"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 6,
                        "prefix": "-scale"
                    },
                    "doc": "Scale the coverage by a constant factor.\nEach coverage value is multiplied by this factor before being reported.\nUseful for normalizing coverage by, e.g., reads per million (RPM).\n- Default is 1.0; i.e., unscaled.\n- (FLOAT)\n",
                    "id": "#bedtools-genomecov.cwl/scale"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 8,
                        "prefix": "-split"
                    },
                    "doc": "treat \"split\" BAM or BED12 entries as distinct BED intervals.\nwhen computing coverage.\nFor BAM files, this uses the CIGAR \"N\" and \"D\" operations\nto infer the blocks for computing coverage.\nFor BED12 files, this uses the BlockCount, BlockStarts, and BlockEnds\nfields (i.e., columns 10,11,12).\n",
                    "id": "#bedtools-genomecov.cwl/split"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 9,
                        "prefix": "-strand"
                    },
                    "doc": "Calculate coverage of intervals from a specific strand.\nWith BED files, requires at least 6 columns (strand is column 6).\n- (STRING): can be + or -\n",
                    "id": "#bedtools-genomecov.cwl/strand"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.output_filename == null){\n    return default_output_filename();\n  } else {\n    return inputs.output_filename;\n  }\n}\n"
                    },
                    "doc": "Generated genome coverage output file\n",
                    "id": "#bedtools-genomecov.cwl/genome_coverage_file"
                }
            ],
            "stdout": "${\n  if (inputs.output_filename == null){\n    return default_output_filename();\n  } else {\n    return inputs.output_filename;\n  }\n}\n",
            "baseCommand": [
                "bedtools",
                "genomecov"
            ],
            "doc": "Tool calculates genome coverage from input bam/bed/gff/vcf using `bedtools genomecov`\n\nDepending on `input_file` extension additional prefix is used: if `*.bam` use `-ibam`, else use `-i`.\n\n`scale` and `mapped_reads_number` inputs result in the same parameter `-scale`. If `scale` is not provided, check if\n`mapped_reads_number` is not null and calculate `-scale` as `1000000/mapped_reads_number`. If both inputs are\nnull, `bedtools genomecov` will use its default scaling value.\n\n`default_output_filename` function returns default output filename and is used when `output_filename` is not provided.\nDefault output file extention is `.tab`. If bedGraph should be generated (check flags `inputs.depth`), extension is\nupdated to `.bedGraph`. Default basename of the output file is generated on the base of `input_file` basename.\n",
            "id": "#bedtools-genomecov.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#bedtools-metadata.yaml",
                "http://schema.org/name": "bowtie2",
                "http://schema.org/about": "A software suite for the comparison, manipulation and annotation of genomic features in browser extensible data (BED) and general feature format (GFF) format. BEDTools also supports the comparison of sequence alignments in BAM format to both BED and GFF features. The tools are extremely efficient and allow the user to compare large datasets (e.g. next-generation sequencing data) with both public and custom genome annotation tracks. BEDTools can be combined with one another as well as with standard UNIX commands, thus facilitating routine genomics tasks as well as pipelines that can quickly answer intricate questions of large genomic datasets.\n",
                "http://schema.org/url": "http://bedtools.readthedocs.org",
                "http://schema.org/codeRepository": "https://github.com/arq5x/bedtools2",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-2.0"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "2.26.0",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#bedtools-metadata.yaml/10.1093/bioinformatics/btq033",
                        "datePublished": "28 January 2010",
                        "http://schema.org/name": "BEDTools: a flexible suite of utilities for comparing genomic features",
                        "http://schema.org/author": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Aaron R. Quinlan"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Ira M. Hall"
                            }
                        ],
                        "http://schema.org/url": "https://academic.oup.com/bioinformatics/article-lookup/doi/10.1093/bioinformatics/btq033"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://github.com/arq5x/bedtools2/issues",
                    "https://groups.google.com/forum/#!forum/bedtools-discuss"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "University of Utah",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Aaron Quinlan",
                                "http://schema.org/description": "Main author"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Neil Kindlon",
                                "http://schema.org/description": "Main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "bedtools-genomecov",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/bedtools-genomecov.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage: bedtools genomecov [OPTIONS] -i <bed/gff/vcf> -g <genome>\n\nOptions:\n\t-ibam\t\tThe input file is in BAM format.\n\t\t\tNote: BAM _must_ be sorted by position\n\n\t-d\t\tReport the depth at each genome position (with one-based coordinates).\n\t\t\tDefault behavior is to report a histogram.\n\n\t-dz\t\tReport the depth at each genome position (with zero-based coordinates).\n\t\t\tReports only non-zero positions.\n\t\t\tDefault behavior is to report a histogram.\n\n\t-bg\t\tReport depth in BedGraph format. For details, see:\n\t\t\tgenome.ucsc.edu/goldenPath/help/bedgraph.html\n\n\t-bga\t\tReport depth in BedGraph format, as above (-bg).\n\t\t\tHowever with this option, regions with zero\n\t\t\tcoverage are also reported. This allows one to\n\t\t\tquickly extract all regions of a genome with 0\n\t\t\tcoverage by applying: \"grep -w 0$\" to the output.\n\n\t-split\t\tTreat \"split\" BAM or BED12 entries as distinct BED intervals.\n\t\t\twhen computing coverage.\n\t\t\tFor BAM files, this uses the CIGAR \"N\" and \"D\" operations\n\t\t\tto infer the blocks for computing coverage.\n\t\t\tFor BED12 files, this uses the BlockCount, BlockStarts, and BlockEnds\n\t\t\tfields (i.e., columns 10,11,12).\n\n\t-strand\t\tCalculate coverage of intervals from a specific strand.\n\t\t\tWith BED files, requires at least 6 columns (strand is column 6).\n\t\t\t- (STRING): can be + or -\n\n\t-pc\t\tCalculate coverage of pair-end fragments.\n\t\t\tWorks for BAM files only\n\t-fs\t\tForce to use provided fragment size instead of read length\n\t\t\tWorks for BAM files only\n\t-du\t\tChange strand af the mate read (so both reads from the same strand) useful for strand specific\n\t\t\tWorks for BAM files only\n\t-5\t\tCalculate coverage of 5\" positions (instead of entire interval).\n\n\t-3\t\tCalculate coverage of 3\" positions (instead of entire interval).\n\n\t-max\t\tCombine all positions with a depth >= max into\n\t\t\ta single bin in the histogram. Irrelevant\n\t\t\tfor -d and -bedGraph\n\t\t\t- (INTEGER)\n\n\t-scale\t\tScale the coverage by a constant factor.\n\t\t\tEach coverage value is multiplied by this factor before being reported.\n\t\t\tUseful for normalizing coverage by, e.g., reads per million (RPM).\n\t\t\t- Default is 1.0; i.e., unscaled.\n\t\t\t- (FLOAT)\n\n\t-trackline\tAdds a UCSC/Genome-Browser track line definition in the first line of the output.\n\t\t\t- See here for more details about track line definition:\n\t\t\t      http://genome.ucsc.edu/goldenPath/help/bedgraph.html\n\t\t\t- NOTE: When adding a trackline definition, the output BedGraph can be easily\n\t\t\t      uploaded to the Genome Browser as a custom track,\n\t\t\t      BUT CAN NOT be converted into a BigWig file (w/o removing the first line).\n\n\t-trackopts\tWrites additional track line definition parameters in the first line.\n\t\t\t- Example:\n\t\t\t   -trackopts 'name=\"My Track\" visibility=2 color=255,30,30'\n\t\t\t   Note the use of single-quotes if you have spaces in your parameters.\n\t\t\t- (TEXT)\n\nNotes:\n\t(1) The genome file should tab delimited and structured as follows:\n\t <chromName><TAB><chromSize>\n\n\tFor example, Human (hg19):\n\tchr1\t249250621\n\tchr2\t243199373\n\t...\n\tchr18_gl000207_random\t4262\n\n\t(2) The input BED (-i) file must be grouped by chromosome.\n\t A simple \"sort -k 1,1 <BED> > <BED>.sorted\" will suffice.\n\n\t(3) The input BAM (-ibam) file must be sorted by position.\n\t A \"samtools sort <BAM>\" should suffice.\n\nTips:\n\tOne can use the UCSC Genome Browser's MySQL database to extract\n\tchromosome sizes. For example, H. sapiens:\n\n\tmysql --user=genome --host=genome-mysql.cse.ucsc.edu -A -e \\\n\t\"select chrom, size from hg19.chromInfo\" > hg19.genome\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { if (Array.isArray(inputs.filelist) && inputs.filelist.length > 0){ return inputs.filelist[0].location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+\".sam\"; } else if (inputs.filelist != null){ return inputs.filelist.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+\".sam\"; } else if (Array.isArray(inputs.filelist_mates) && inputs.filelist_mates.length > 0){ return inputs.filelist_mates[0].location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+\".sam\"; } else if (inputs.filelist_mates != null){ return inputs.filelist_mates.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+\".sam\"; } else { return null; } };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/bowtie2:v2.3.0",
                    "dockerFile": "$import: ./dockerfiles/bowtie2-Dockerfile\n"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "report all alignments; very slow, MAPQ not meaningful\n",
                    "inputBinding": {
                        "position": 27,
                        "prefix": "-a"
                    },
                    "id": "#bowtie2.cwl/a"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "write unpaired reads that aligned at least once to <path>\n",
                    "inputBinding": {
                        "position": 39,
                        "prefix": "--al"
                    },
                    "id": "#bowtie2.cwl/al"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "write pairs that aligned concordantly at least once to <path>\n",
                    "inputBinding": {
                        "position": 41,
                        "prefix": "--al-conc"
                    },
                    "id": "#bowtie2.cwl/al_conc"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "<m1>, <m2>, <r> are sequences themselves, not files",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-c"
                    },
                    "id": "#bowtie2.cwl/c"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "trim <int> bases from 3'/right end of reads (0)\n",
                    "inputBinding": {
                        "position": 5,
                        "prefix": "-3"
                    },
                    "id": "#bowtie2.cwl/clip_3p_end"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "trim <int> bases from 5'/left end of reads (0)\n",
                    "inputBinding": {
                        "position": 4,
                        "prefix": "-5"
                    },
                    "id": "#bowtie2.cwl/clip_5p_end"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "give up extending after <int> failed extends in a row (15)\n",
                    "inputBinding": {
                        "position": 28,
                        "prefix": "-D"
                    },
                    "id": "#bowtie2.cwl/d"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "include <int> extra ref chars on sides of DP table (15)\n",
                    "inputBinding": {
                        "position": 11,
                        "prefix": "--dpad"
                    },
                    "id": "#bowtie2.cwl/dpad"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "entire read must align; no clipping (on)\nOptions:\n  --very-fast            -D 5 -R 1 -N 0 -L 22 -i S,0,2.50\n  --fast                 -D 10 -R 2 -N 0 -L 22 -i S,0,2.50\n  --sensitive            -D 15 -R 2 -N 0 -L 22 -i S,1,1.15 (default)\n  --very-sensitive       -D 20 -R 3 -N 0 -L 20 -i S,1,0.50\n",
                    "inputBinding": {
                        "position": 17,
                        "prefix": "--end-to-end"
                    },
                    "id": "#bowtie2.cwl/end_to_end"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Option for end_to_end:\n  --very-fast            -D 5 -R 1 -N 0 -L 22 -i S,0,2.50\n",
                    "inputBinding": {
                        "position": 18,
                        "prefix": "--fast"
                    },
                    "id": "#bowtie2.cwl/end_to_end_fast"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Option for end_to_end:\n  --sensitive            -D 15 -R 2 -N 0 -L 22 -i S,1,1.15 (default)\n",
                    "inputBinding": {
                        "position": 18,
                        "prefix": "--sensitive"
                    },
                    "id": "#bowtie2.cwl/end_to_end_sensitive"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Option for end_to_end:\n  --very-fast            -D 5 -R 1 -N 0 -L 22 -i S,0,2.50\n",
                    "inputBinding": {
                        "position": 18,
                        "prefix": "--very-fast"
                    },
                    "id": "#bowtie2.cwl/end_to_end_very_fast"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Option for end_to_end:\n  --very-sensitive       -D 20 -R 3 -N 0 -L 20 -i S,1,0.50\n",
                    "inputBinding": {
                        "position": 18,
                        "prefix": "--very-sensitive"
                    },
                    "id": "#bowtie2.cwl/end_to_end_very_sensitive"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "query input files are (multi-)FASTA .fa/.mfa",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-f"
                    },
                    "id": "#bowtie2.cwl/f"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "-1, -2 mates align fw/rev, rev/fw, fw/fw (--fr)\n",
                    "inputBinding": {
                        "position": 31,
                        "prefix": "--ff"
                    },
                    "id": "#bowtie2.cwl/ff"
                },
                {
                    "type": [
                        "null",
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "doc": "{-1 <m1> -2 <m2> | -U <r>} [-S <sam>]\n<m1>       Files with #1 mates, paired with files in <m2>.\n           Could be gzip'ed (extension: .gz) or bzip2'ed (extension: .bz2).\n<m2>       Files with #2 mates, paired with files in <m1>.\n           Could be gzip'ed (extension: .gz) or bzip2'ed (extension: .bz2).\n<r>        Files with unpaired reads.\n           Could be gzip'ed (extension: .gz) or bzip2'ed (extension: .bz2).\n",
                    "inputBinding": {
                        "itemSeparator": ",",
                        "position": 83
                    },
                    "id": "#bowtie2.cwl/filelist"
                },
                {
                    "type": [
                        "null",
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "itemSeparator": ",",
                        "position": 85
                    },
                    "id": "#bowtie2.cwl/filelist_mates"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "-1, -2 mates align fw/rev, rev/fw, fw/fw (--fr)\n",
                    "inputBinding": {
                        "position": 31,
                        "prefix": "--fr"
                    },
                    "id": "#bowtie2.cwl/fr"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "disallow gaps within <int> nucs of read extremes (4)\n",
                    "inputBinding": {
                        "position": 12,
                        "prefix": "--gbar"
                    },
                    "id": "#bowtie2.cwl/gbar"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "interval between seed substrings w/r/t read len (S,1,1.15)\n",
                    "inputBinding": {
                        "position": 9,
                        "prefix": "-i"
                    },
                    "id": "#bowtie2.cwl/i"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "treat all quality values as 30 on Phred scale (off)\n",
                    "inputBinding": {
                        "position": 13,
                        "prefix": "--ignore-quals"
                    },
                    "id": "#bowtie2.cwl/ignore_quals"
                },
                {
                    "type": "Directory",
                    "doc": "Folder with indices files\n",
                    "inputBinding": {
                        "position": 81,
                        "prefix": "-x",
                        "valueFrom": "${\n    for (var i = 0; i < self.listing.length; i++) {\n        if (self.listing[i].path.split('.').slice(-3).join('.') == 'rev.1.bt2' ||\n            self.listing[i].path.split('.').slice(-3).join('.') == 'rev.1.bt2l'){\n          return self.listing[i].path.split('.').slice(0,-3).join('.');\n        }\n    }\n    return null;\n}\n"
                    },
                    "id": "#bowtie2.cwl/indices_folder"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "qualities encoded as space-delimited integers",
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--int-quals"
                    },
                    "id": "#bowtie2.cwl/integer_quals"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "report up to <int> alns per read; MAPQ not meaningful\n",
                    "inputBinding": {
                        "position": 27,
                        "prefix": "-k"
                    },
                    "id": "#bowtie2.cwl/k"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "length of seed substrings; must be >3, <32 (22)\n",
                    "inputBinding": {
                        "position": 8,
                        "prefix": "-L"
                    },
                    "id": "#bowtie2.cwl/l"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "local alignment; ends might be soft clipped (off)\nOptions:\n  --very-fast-local      -D 5 -R 1 -N 0 -L 25 -i S,1,2.00\n  --fast-local           -D 10 -R 2 -N 0 -L 22 -i S,1,1.75\n  --sensitive-local      -D 15 -R 2 -N 0 -L 20 -i S,1,0.75 (default)\n  --very-sensitive-local -D 20 -R 3 -N 0 -L 20 -i S,1,0.50\n",
                    "inputBinding": {
                        "position": 19,
                        "prefix": "--local"
                    },
                    "id": "#bowtie2.cwl/local"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Option for local:\n  --fast-local           -D 10 -R 2 -N 0 -L 22 -i S,1,1.75\n",
                    "inputBinding": {
                        "position": 20,
                        "prefix": "--fast-local"
                    },
                    "id": "#bowtie2.cwl/local_fast_local"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Option for local:\n  --sensitive-local      -D 15 -R 2 -N 0 -L 20 -i S,1,0.75 (default)\n",
                    "inputBinding": {
                        "position": 20,
                        "prefix": "--sensitive-local"
                    },
                    "id": "#bowtie2.cwl/local_sensitive_local"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Option for local:\n  --very-fast-local      -D 5 -R 1 -N 0 -L 25 -i S,1,2.00\n",
                    "inputBinding": {
                        "position": 20,
                        "prefix": "--very-fast-local"
                    },
                    "id": "#bowtie2.cwl/local_very_fast_local"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Option for local:\n  --very-sensitive-local -D 20 -R 3 -N 0 -L 20 -i S,1,0.50\n",
                    "inputBinding": {
                        "position": 20,
                        "prefix": "--very-sensitive-local"
                    },
                    "id": "#bowtie2.cwl/local_very_sensitive_local"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "match bonus (0 for --end-to-end, 2 for --local)\n",
                    "inputBinding": {
                        "position": 21,
                        "prefix": "--ma"
                    },
                    "id": "#bowtie2.cwl/ma"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "maxins fragment length (0)\n",
                    "inputBinding": {
                        "position": 30,
                        "prefix": "--maxins"
                    },
                    "id": "#bowtie2.cwl/maxins"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "report internal counters & metrics every <int> secs (1)\n",
                    "inputBinding": {
                        "position": 45,
                        "prefix": "--met"
                    },
                    "id": "#bowtie2.cwl/met"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "send metrics to file at <path> (off)\n",
                    "inputBinding": {
                        "position": 43,
                        "prefix": "--met-file"
                    },
                    "id": "#bowtie2.cwl/met_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "send metrics to stderr (off)",
                    "inputBinding": {
                        "position": 44,
                        "prefix": "--met-stderr"
                    },
                    "id": "#bowtie2.cwl/met_stderr"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "minimum fragment length (0)\n",
                    "inputBinding": {
                        "position": 30,
                        "prefix": "--minins"
                    },
                    "id": "#bowtie2.cwl/minins"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "use memory-mapped I/O for index; many 'bowtie's can share",
                    "inputBinding": {
                        "position": 54,
                        "prefix": "--mm"
                    },
                    "id": "#bowtie2.cwl/mm"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "max penalty for mismatch; lower qual = lower penalty (6)\n",
                    "inputBinding": {
                        "position": 22,
                        "prefix": "--mp"
                    },
                    "id": "#bowtie2.cwl/mp"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "max # mismatches in seed alignment; can be 0 or 1 (0)\n",
                    "inputBinding": {
                        "position": 7,
                        "prefix": "-N"
                    },
                    "id": "#bowtie2.cwl/n"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "func for max # non-A/C/G/Ts permitted in aln (L,0,0.15)\n",
                    "inputBinding": {
                        "position": 10,
                        "prefix": "--n-ceil"
                    },
                    "id": "#bowtie2.cwl/n_ceil"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "do not allow 1 mismatch alignments before attempting to scan for the optimal seeded alignments\n",
                    "inputBinding": {
                        "position": 16,
                        "prefix": "--no-1mm-upfront"
                    },
                    "id": "#bowtie2.cwl/no_1mm_upfront"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "not concordant when one mate alignment contains other\n",
                    "inputBinding": {
                        "position": 35,
                        "prefix": "--no-contain"
                    },
                    "id": "#bowtie2.cwl/no_contain"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "suppress discordant alignments for paired reads\n",
                    "inputBinding": {
                        "position": 33,
                        "prefix": "--no-discordant"
                    },
                    "id": "#bowtie2.cwl/no_discordant"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "not concordant when mates extend past each other\n",
                    "inputBinding": {
                        "position": 34,
                        "prefix": "--no-dovetail"
                    },
                    "id": "#bowtie2.cwl/no_dovetail"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "suppress header lines, i.e. lines starting with @",
                    "inputBinding": {
                        "position": 47,
                        "prefix": "--no-head"
                    },
                    "id": "#bowtie2.cwl/no_head"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "suppress unpaired alignments for paired reads\n",
                    "inputBinding": {
                        "position": 32,
                        "prefix": "--no-mixed"
                    },
                    "id": "#bowtie2.cwl/no_mixed"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "not concordant when mates overlap at all\n",
                    "inputBinding": {
                        "position": 36,
                        "prefix": "--no-overlap"
                    },
                    "id": "#bowtie2.cwl/no_overlap"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "suppress @SQ header lines",
                    "inputBinding": {
                        "position": 48,
                        "prefix": "--no-sq"
                    },
                    "id": "#bowtie2.cwl/no_sq"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "suppress SAM records for unaligned reads",
                    "inputBinding": {
                        "position": 46,
                        "prefix": "--no-unal"
                    },
                    "id": "#bowtie2.cwl/no_unal"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "do not align forward (original) version of read (off)\n",
                    "inputBinding": {
                        "position": 14,
                        "prefix": "--nofw"
                    },
                    "id": "#bowtie2.cwl/nofw"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "seed rand. gen. arbitrarily instead of using read attributes",
                    "inputBinding": {
                        "position": 57,
                        "prefix": "--non-deterministic"
                    },
                    "id": "#bowtie2.cwl/non_deterministic"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "do not align reverse-complement version of read (off)\n",
                    "inputBinding": {
                        "position": 15,
                        "prefix": "--norc"
                    },
                    "id": "#bowtie2.cwl/norc"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "penalty for non-A/C/G/Ts in read/ref (1)\n",
                    "inputBinding": {
                        "position": 23,
                        "prefix": "--np"
                    },
                    "id": "#bowtie2.cwl/np"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "put '*' in SEQ and QUAL fields for secondary alignments",
                    "inputBinding": {
                        "position": 51,
                        "prefix": "--omit-sec-seq"
                    },
                    "id": "#bowtie2.cwl/omit_sec_seq"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 90,
                        "prefix": "-S",
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "File for SAM output (default: stdout)\n",
                    "id": "#bowtie2.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "qualities are Phred+33 (default)",
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--phred33"
                    },
                    "id": "#bowtie2.cwl/phred33_quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "qualities are Phred+64",
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--phred64"
                    },
                    "id": "#bowtie2.cwl/phred64_quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "query input files are FASTQ .fq/.fastq (default)",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-q"
                    },
                    "id": "#bowtie2.cwl/q"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "filter out reads that are bad according to QSEQ filter",
                    "inputBinding": {
                        "position": 55,
                        "prefix": "--qc-filter"
                    },
                    "id": "#bowtie2.cwl/qc_filter"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "query input files are in Illumina's qseq format",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--qseq"
                    },
                    "id": "#bowtie2.cwl/qseq"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "print nothing to stderr except serious errors",
                    "inputBinding": {
                        "position": 42,
                        "prefix": "--quiet"
                    },
                    "id": "#bowtie2.cwl/quiet"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "for reads w/ repetitive seeds, try <int> sets of seeds (2)\n",
                    "inputBinding": {
                        "position": 29,
                        "prefix": "-R"
                    },
                    "id": "#bowtie2.cwl/r"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "query input files are raw one-sequence-per-line",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-r"
                    },
                    "id": "#bowtie2.cwl/raw"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "int"
                        }
                    ],
                    "doc": "read gap open, extend penalties (5,3)\n",
                    "inputBinding": {
                        "position": 24,
                        "itemSeparator": ",",
                        "prefix": "--rdg"
                    },
                    "id": "#bowtie2.cwl/rdg"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "force SAM output order to match order of input reads\n",
                    "inputBinding": {
                        "position": 53,
                        "prefix": "--reorder"
                    },
                    "id": "#bowtie2.cwl/reorder"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "-1, -2 mates align fw/rev, rev/fw, fw/fw (--fr)\n",
                    "inputBinding": {
                        "position": 31,
                        "prefix": "--rf"
                    },
                    "id": "#bowtie2.cwl/rf"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "int"
                        }
                    ],
                    "doc": "reference gap open, extend penalties (5,3)\n",
                    "inputBinding": {
                        "position": 25,
                        "itemSeparator": ",",
                        "prefix": "--rfg"
                    },
                    "id": "#bowtie2.cwl/rfg"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "add <text> (\"lab:value\") to @RG line of SAM header.\nNote: @RG line only printed when --rg-id is set.\n",
                    "inputBinding": {
                        "position": 50,
                        "prefix": "--rg"
                    },
                    "id": "#bowtie2.cwl/rg"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "set read group id, reflected in @RG line and RG:Z: opt field",
                    "inputBinding": {
                        "position": 49,
                        "prefix": "--rg-id"
                    },
                    "id": "#bowtie2.cwl/rg_id"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "skip the first <int> reads/pairs in the input (none)\n",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-s"
                    },
                    "id": "#bowtie2.cwl/s"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "min acceptable alignment score w/r/t read length (G,20,8 for local, L,-0.6,-0.6 for end-to-end)\n",
                    "inputBinding": {
                        "position": 26,
                        "prefix": "--score-min"
                    },
                    "id": "#bowtie2.cwl/score_min"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "seed for random number generator (0)\n",
                    "inputBinding": {
                        "position": 56,
                        "prefix": "--seed"
                    },
                    "id": "#bowtie2.cwl/seed"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "print wall-clock time taken by search phases\n",
                    "inputBinding": {
                        "position": 37,
                        "prefix": "-t"
                    },
                    "id": "#bowtie2.cwl/t"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "number of alignment threads to launch (1)\n",
                    "inputBinding": {
                        "position": 52,
                        "prefix": "-p"
                    },
                    "id": "#bowtie2.cwl/threads"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "stop after first <int> reads/pairs (no limit)\n",
                    "inputBinding": {
                        "position": 3,
                        "prefix": "-u"
                    },
                    "id": "#bowtie2.cwl/u"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "write unpaired reads that didn't align to <path>\n",
                    "inputBinding": {
                        "position": 38,
                        "prefix": "--un"
                    },
                    "id": "#bowtie2.cwl/un"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "write pairs that didn't align concordantly to <path>\n",
                    "inputBinding": {
                        "position": 40,
                        "prefix": "--un-conc"
                    },
                    "id": "#bowtie2.cwl/un_conc"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n   if (inputs.output_filename == \"\"){\n     return default_output_filename();\n   } else {\n     return inputs.output_filename;\n   }\n}\n"
                    },
                    "id": "#bowtie2.cwl/output"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n   if (inputs.output_filename == \"\"){\n     return default_output_filename().split('.').slice(0,-1).join('.') + \".log\";\n   } else {\n     return inputs.output_filename.split('.').slice(0,-1).join('.') + \".log\";\n   }\n}\n"
                    },
                    "id": "#bowtie2.cwl/output_log"
                }
            ],
            "baseCommand": [
                "bowtie2"
            ],
            "arguments": [
                {
                    "valueFrom": "${\n  if (inputs.filelist && inputs.filelist_mates){\n    return \"-1\";\n  } else if (inputs.filelist){\n    return \"-U\";\n  } else {\n    return null;\n  }\n}\n",
                    "position": 82
                },
                {
                    "valueFrom": "${\n  if (inputs.filelist && inputs.filelist_mates){\n    return \"-2\";\n  } else if (inputs.filelist_mates){\n    return \"-U\";\n  } else {\n    return null;\n  }\n}\n",
                    "position": 84
                },
                {
                    "valueFrom": "${\n  if (inputs.output_filename == \"\"){\n    return ' 2> ' + default_output_filename().split('.').slice(0,-1).join('.') + '.log';\n  } else {\n    return ' 2> ' + inputs.output_filename.split('.').slice(0,-1).join('.') + '.log';\n  }\n}\n",
                    "position": 100000,
                    "shellQuote": false
                }
            ],
            "doc": "Tool is used to run bowtie aligner to align input FASTQ file(s) to reference genome\n",
            "id": "#bowtie2.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#bowtie2-metadata.yaml",
                "http://schema.org/name": "bowtie2",
                "http://schema.org/about": "Bowtie 2 is an ultrafast and memory-efficient tool for aligning sequencing reads to long reference sequences. It is particularly good at aligning reads of about 50 up to 100s or 1,000s of characters, and particularly good at aligning to relatively long (e.g. mammalian) genomes. Bowtie 2 indexes the genome with an FM Index to keep its memory footprint small: for the human genome, its memory footprint is typically around 3.2 GB. Bowtie 2 supports gapped, local, and paired-end alignment modes.\n",
                "http://schema.org/url": "http://bowtie-bio.sourceforge.net/bowtie2/index.shtml",
                "http://schema.org/codeRepository": "https://github.com/BenLangmead/bowtie2",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "2.3.0",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#bowtie2-metadata.yaml/10.1038/nmeth.1923",
                        "datePublished": "04 March 2012",
                        "http://schema.org/name": "Fast gapped-read alignment with Bowtie 2",
                        "http://schema.org/author": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Ben Langmead"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Steven L Salzberg"
                            }
                        ],
                        "http://schema.org/url": "http://www.nature.com/nmeth/journal/v9/n4/full/nmeth.1923.html"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://github.com/BenLangmead/bowtie2/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Johns Hopkins University",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Ben Langmead",
                                "http://schema.org/description": "Main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "bowtie2",
            "http://schema.org/downloadUrl": "https://github.com/SciDAP/workflows/blob/master/tools/bowtie2.cwl",
            "http://schema.org/codeRepository": "https://github.com/SciDAP/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:michael.kotliar@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage:\n  bowtie2 [options]* -x <bt2-idx> {-1 <m1> -2 <m2> | -U <r>} [-S <sam>]\n\n  <bt2-idx>  Index filename prefix (minus trailing .X.bt2).\n             NOTE: Bowtie 1 and Bowtie 2 indexes are not compatible.\n  <m1>       Files with #1 mates, paired with files in <m2>.\n             Could be gzip'ed (extension: .gz) or bzip2'ed (extension: .bz2).\n  <m2>       Files with #2 mates, paired with files in <m1>.\n             Could be gzip'ed (extension: .gz) or bzip2'ed (extension: .bz2).\n  <r>        Files with unpaired reads.\n             Could be gzip'ed (extension: .gz) or bzip2'ed (extension: .bz2).\n  <sam>      File for SAM output (default: stdout)\n\n  <m1>, <m2>, <r> can be comma-separated lists (no whitespace) and can be\n  specified many times.  E.g. '-U file1.fq,file2.fq -U file3.fq'.\n\nOptions (defaults in parentheses):\n\n Input:\n  -q                 query input files are FASTQ .fq/.fastq (default)\n  --qseq             query input files are in Illumina's qseq format\n  -f                 query input files are (multi-)FASTA .fa/.mfa\n  -r                 query input files are raw one-sequence-per-line\n  -c                 <m1>, <m2>, <r> are sequences themselves, not files\n  -s/--skip <int>    skip the first <int> reads/pairs in the input (none)\n  -u/--upto <int>    stop after first <int> reads/pairs (no limit)\n  -5/--trim5 <int>   trim <int> bases from 5'/left end of reads (0)\n  -3/--trim3 <int>   trim <int> bases from 3'/right end of reads (0)\n  --phred33          qualities are Phred+33 (default)\n  --phred64          qualities are Phred+64\n  --int-quals        qualities encoded as space-delimited integers\n\n Presets:                 Same as:\n  For --end-to-end:\n   --very-fast            -D 5 -R 1 -N 0 -L 22 -i S,0,2.50\n   --fast                 -D 10 -R 2 -N 0 -L 22 -i S,0,2.50\n   --sensitive            -D 15 -R 2 -N 0 -L 22 -i S,1,1.15 (default)\n   --very-sensitive       -D 20 -R 3 -N 0 -L 20 -i S,1,0.50\n\n  For --local:\n   --very-fast-local      -D 5 -R 1 -N 0 -L 25 -i S,1,2.00\n   --fast-local           -D 10 -R 2 -N 0 -L 22 -i S,1,1.75\n   --sensitive-local      -D 15 -R 2 -N 0 -L 20 -i S,1,0.75 (default)\n   --very-sensitive-local -D 20 -R 3 -N 0 -L 20 -i S,1,0.50\n\n Alignment:\n  -N <int>           max # mismatches in seed alignment; can be 0 or 1 (0)\n  -L <int>           length of seed substrings; must be >3, <32 (22)\n  -i <func>          interval between seed substrings w/r/t read len (S,1,1.15)\n  --n-ceil <func>    func for max # non-A/C/G/Ts permitted in aln (L,0,0.15)\n  --dpad <int>       include <int> extra ref chars on sides of DP table (15)\n  --gbar <int>       disallow gaps within <int> nucs of read extremes (4)\n  --ignore-quals     treat all quality values as 30 on Phred scale (off)\n  --nofw             do not align forward (original) version of read (off)\n  --norc             do not align reverse-complement version of read (off)\n  --no-1mm-upfront   do not allow 1 mismatch alignments before attempting to\n                     scan for the optimal seeded alignments\n  --end-to-end       entire read must align; no clipping (on)\n   OR\n  --local            local alignment; ends might be soft clipped (off)\n\n Scoring:\n  --ma <int>         match bonus (0 for --end-to-end, 2 for --local)\n  --mp <int>         max penalty for mismatch; lower qual = lower penalty (6)\n  --np <int>         penalty for non-A/C/G/Ts in read/ref (1)\n  --rdg <int>,<int>  read gap open, extend penalties (5,3)\n  --rfg <int>,<int>  reference gap open, extend penalties (5,3)\n  --score-min <func> min acceptable alignment score w/r/t read length\n                     (G,20,8 for local, L,-0.6,-0.6 for end-to-end)\n\n Reporting:\n  (default)          look for multiple alignments, report best, with MAPQ\n   OR\n  -k <int>           report up to <int> alns per read; MAPQ not meaningful\n   OR\n  -a/--all           report all alignments; very slow, MAPQ not meaningful\n\n Effort:\n  -D <int>           give up extending after <int> failed extends in a row (15)\n  -R <int>           for reads w/ repetitive seeds, try <int> sets of seeds (2)\n\n Paired-end:\n  -I/--minins <int>  minimum fragment length (0)\n  -X/--maxins <int>  maximum fragment length (500)\n  --fr/--rf/--ff     -1, -2 mates align fw/rev, rev/fw, fw/fw (--fr)\n  --no-mixed         suppress unpaired alignments for paired reads\n  --no-discordant    suppress discordant alignments for paired reads\n  --no-dovetail      not concordant when mates extend past each other\n  --no-contain       not concordant when one mate alignment contains other\n  --no-overlap       not concordant when mates overlap at all\n\n Output:\n  -t/--time          print wall-clock time taken by search phases\n  --un <path>           write unpaired reads that didn't align to <path>\n  --al <path>           write unpaired reads that aligned at least once to <path>\n  --un-conc <path>      write pairs that didn't align concordantly to <path>\n  --al-conc <path>      write pairs that aligned concordantly at least once to <path>\n  (Note: for --un, --al, --un-conc, or --al-conc, add '-gz' to the option name, e.g.\n  --un-gz <path>, to gzip compress output, or add '-bz2' to bzip2 compress output.)\n  --quiet            print nothing to stderr except serious errors\n  --met-file <path>  send metrics to file at <path> (off)\n  --met-stderr       send metrics to stderr (off)\n  --met <int>        report internal counters & metrics every <int> secs (1)\n  --no-unal          suppress SAM records for unaligned reads\n  --no-head          suppress header lines, i.e. lines starting with @\n  --no-sq            suppress @SQ header lines\n  --rg-id <text>     set read group id, reflected in @RG line and RG:Z: opt field\n  --rg <text>        add <text> (\"lab:value\") to @RG line of SAM header.\n                     Note: @RG line only printed when --rg-id is set.\n  --omit-sec-seq     put '*' in SEQ and QUAL fields for secondary alignments.\n\n Performance:\n  -p/--threads <int> number of alignment threads to launch (1)\n  --reorder          force SAM output order to match order of input reads\n  --mm               use memory-mapped I/O for index; many 'bowtie's can share\n\n Other:\n  --qc-filter        filter out reads that are bad according to QSEQ filter\n  --seed <int>       seed for random number generator (0)\n  --non-deterministic seed rand. gen. arbitrarily instead of using read attributes\n  --version          print version information and quit\n  -h/--help          print this usage message\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { return inputs.unsorted_file.location.split('/').slice(-1)[0]; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/scidap:v0.0.2"
                }
            ],
            "inputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "string",
                        "inputBinding": {
                            "prefix": "-k"
                        }
                    },
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "-k, --key=POS1[,POS2]\nstart a key at POS1, end it at POS2 (origin 1)\n",
                    "id": "#linux-sort.cwl/key"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Name for generated output file\n",
                    "id": "#linux-sort.cwl/output_filename"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 4
                    },
                    "id": "#linux-sort.cwl/unsorted_file"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.output_filename == null){\n    return default_output_filename();\n  } else {\n    return inputs.output_filename;\n  }\n}\n"
                    },
                    "doc": "Sorted file\n",
                    "id": "#linux-sort.cwl/sorted_file"
                }
            ],
            "stdout": "${\n  if (inputs.output_filename == null){\n    return default_output_filename();\n  } else {\n    return inputs.output_filename;\n  }\n}\n",
            "baseCommand": [
                "sort"
            ],
            "doc": "Tool sorts data from `unsorted_file` by key\n\n`default_output_filename` function returns file name identical to `unsorted_file`, if `output_filename` is not provided.\n",
            "id": "#linux-sort.cwl",
            "http://schema.org/name": "linux-sort",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/linux-sort.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { return inputs.input_file.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+\".bam\"; };",
                        "var default_output_metrics_filename = function() { return inputs.input_file.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+\".metrics\"; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/picard:v2.9.0",
                    "dockerFile": "$import: ./dockerfiles/picard-Dockerfile\n"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "name": "#picard-markduplicates.cwl/assume_sort_order/assume_sort_order",
                            "symbols": [
                                "#picard-markduplicates.cwl/assume_sort_order/assume_sort_order/unsorted",
                                "#picard-markduplicates.cwl/assume_sort_order/assume_sort_order/queryname",
                                "#picard-markduplicates.cwl/assume_sort_order/assume_sort_order/coordinate",
                                "#picard-markduplicates.cwl/assume_sort_order/assume_sort_order/duplicate"
                            ]
                        }
                    ],
                    "doc": "If not null, assume that the input file has this order even if the header says otherwise.\nPossible values: {unsorted, queryname, coordinate, duplicate}\nDefault value: null.\n",
                    "inputBinding": {
                        "position": 17,
                        "prefix": "ASSUME_SORT_ORDER=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/assume_sort_order"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Barcode SAM tag (ex. BC for 10X Genomics)\nDefault value: null.\n",
                    "inputBinding": {
                        "position": 8,
                        "prefix": "BARCODE_TAG=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/barcode_tag"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Comment(s) to include in the output file's header.\nDefault value: null.\n",
                    "inputBinding": {
                        "position": 23,
                        "prefix": "COMMENT=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/comment"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "name": "#picard-markduplicates.cwl/dup_scoring_strategy/dup_scoring_strategy",
                            "symbols": [
                                "#picard-markduplicates.cwl/dup_scoring_strategy/dup_scoring_strategy/SUM_OF_BASE_QUALITIES",
                                "#picard-markduplicates.cwl/dup_scoring_strategy/dup_scoring_strategy/TOTAL_MAPPED_REFERENCE_LENGTH",
                                "#picard-markduplicates.cwl/dup_scoring_strategy/dup_scoring_strategy/RANDOM"
                            ]
                        }
                    ],
                    "doc": "The scoring strategy for choosing the non-duplicate among candidates.\nPossible values: {SUM_OF_BASE_QUALITIES, TOTAL_MAPPED_REFERENCE_LENGTH, RANDOM}\nDefault value: SUM_OF_BASE_QUALITIES.\n",
                    "inputBinding": {
                        "position": 18,
                        "prefix": "DUPLICATE_SCORING_STRATEGY=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/dup_scoring_strategy"
                },
                {
                    "type": [
                        "File"
                    ],
                    "doc": "One or more input SAM or BAM files to analyze.\nMust be coordinate sorted.\n",
                    "inputBinding": {
                        "position": 13,
                        "prefix": "INPUT=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/input_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "shellQuote": false
                    },
                    "doc": "JVM arguments should be a quoted, space separated list (e.g. \"-Xms128m -Xmx512m\")\n",
                    "id": "#picard-markduplicates.cwl/java_opts"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "Maximum number of file handles to keep open when spilling read ends to disk.\nSet this number a little lower than the per-process maximum number of file that may be open.\nThis number can be found by executing the 'ulimit -n' command on a Unix system.\nDefault value: 8000.\n",
                    "inputBinding": {
                        "position": 6,
                        "prefix": "MAX_FILE_HANDLES_FOR_READ_ENDS_MAP=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/max_file_handles_for_read_ends_map"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "This option is obsolete. ReadEnds will always be spilled to disk.\nDefault value: 50000.\n",
                    "inputBinding": {
                        "position": 5,
                        "prefix": "MAX_SEQUENCES_FOR_DISK_READ_ENDS_MAP=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/max_seq_for_disk_read_ends_map"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "The maximum offset between two duplicate clusters in order to consider them optical duplicates.\nThe default is appropriate for unpatterned versions of the Illumina platform.\nFor the patterned flowcell models, 2500 is moreappropriate.\nFor other platforms and models, users should experiment to find what works best.\nDefault value: 100.\n",
                    "inputBinding": {
                        "position": 25,
                        "prefix": "OPTICAL_DUPLICATE_PIXEL_DISTANCE=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/optical_dup_pixel_dist"
                },
                {
                    "type": [
                        "string"
                    ],
                    "doc": "The output file to write marked records to Required.\n",
                    "inputBinding": {
                        "position": 14,
                        "prefix": "OUTPUT=",
                        "separate": false,
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "id": "#picard-markduplicates.cwl/output_filename"
                },
                {
                    "type": [
                        "string"
                    ],
                    "doc": "File to write duplication metrics to Required.\n",
                    "inputBinding": {
                        "position": 15,
                        "prefix": "METRICS_FILE=",
                        "separate": false,
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_metrics_filename();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "id": "#picard-markduplicates.cwl/output_metrics_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Value of CL tag of PG record to be created.\nIf not supplied the command line will be detected automatically.\nDefault value: null.\n",
                    "inputBinding": {
                        "position": 21,
                        "prefix": "PROGRAM_GROUP_COMMAND_LINE=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/program_group_cmd_line"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Value of PN tag of PG record to be created.\nDefault value: MarkDuplicates.\n",
                    "inputBinding": {
                        "position": 22,
                        "prefix": "PROGRAM_GROUP_NAME=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/program_group_name"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Value of VN tag of PG record to be created.\nIf not specified, the version will be detected automatically.\nDefault value: null.\n",
                    "inputBinding": {
                        "position": 20,
                        "prefix": "PROGRAM_GROUP_VERSION=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/program_group_version"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "The program record ID for the @PG record(s) created by this program.\nThis string may have a suffix appended to avoid collision with other program record IDs.\nDefault value: MarkDuplicates.\n",
                    "inputBinding": {
                        "position": 19,
                        "prefix": "PROGRAM_RECORD_ID=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/program_record_id"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Regular expression that can be used to parse read names in the incoming SAM file.\nRead names are parsed to extract three variables: tile/region, x coordinate and y coordinate.\nThese values are used to estimate the rate of optical duplication in order to give a more accurate\nestimated library size.\nThe regular expression should contain three capture groups for the three variables, in order.\nIt must match the entire read name. Note that if the default regex is specified, a regex match is not\nactually done, but instead the read name is split on colon character. For 5 element names, the 3rd,\n4th and 5th elements are assumed to be tile, x and y values. For 7 element names (CASAVA 1.8),\nthe 5th, 6th, and 7th elements are assumed to be tile, x and y values.\nDefault value: .\n",
                    "inputBinding": {
                        "position": 24,
                        "prefix": "READ_NAME_REGEX=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/read_name_regex"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Read one barcode SAM tag (ex. BX for 10X Genomics)\nDefault value: null.\n",
                    "inputBinding": {
                        "position": 9,
                        "prefix": "READ_ONE_BARCODE_TAG=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/read_one_barcode_tag"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Read two barcode SAM tag (ex. BX for 10X Genomics).\nDefault value: null.\n",
                    "inputBinding": {
                        "position": 10,
                        "prefix": "READ_TWO_BARCODE_TAG=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/read_two_barcode_tag"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "If true do not write duplicates to the output file instead of writing them with appropriate flags set.\nDefault value: false.\n",
                    "inputBinding": {
                        "position": 16,
                        "prefix": "REMOVE_DUPLICATES=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/remove_dup"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "If true remove 'optical' duplicates and other duplicates that appear to have arisen from the sequencing process\ninstead of the library preparation process, even if REMOVE_DUPLICATES is false. If REMOVE_DUPLICATES is true,\nall duplicates are removed and this option is ignored.\nDefault value: false.\n",
                    "inputBinding": {
                        "position": 11,
                        "prefix": "REMOVE_SEQUENCING_DUPLICATES=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/remove_seq_dup"
                },
                {
                    "type": [
                        "null",
                        "double"
                    ],
                    "doc": "This number, plus the maximum RAM available to the JVM, determine the memory footprint used by some\nof the sorting collections. If you are running out of memory, try reducing this number.\nDefault value: 0.25.\n",
                    "inputBinding": {
                        "position": 7,
                        "prefix": "SORTING_COLLECTION_SIZE_RATIO=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/sorting_coll_size_ratio"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "name": "#picard-markduplicates.cwl/tagging_policy/tagging_policy",
                            "symbols": [
                                "#picard-markduplicates.cwl/tagging_policy/tagging_policy/DontTag",
                                "#picard-markduplicates.cwl/tagging_policy/tagging_policy/OpticalOnly",
                                "#picard-markduplicates.cwl/tagging_policy/tagging_policy/All"
                            ]
                        }
                    ],
                    "doc": "Determines how duplicate types are recorded in the DT optional attribute.\nPossible values: {DontTag, OpticalOnly, All}\nDefault value: DontTag.\n",
                    "inputBinding": {
                        "position": 12,
                        "prefix": "TAGGING_POLICY=",
                        "separate": false
                    },
                    "id": "#picard-markduplicates.cwl/tagging_policy"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.output_metrics_filename == \"\"){\n      return default_output_metrics_filename();\n    } else {\n      return inputs.output_metrics_filename;\n    }\n}\n"
                    },
                    "id": "#picard-markduplicates.cwl/metrics_file"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.output_filename == \"\"){\n      return default_output_filename();\n    } else {\n      return inputs.output_filename;\n    }\n}\n"
                    },
                    "id": "#picard-markduplicates.cwl/output_file"
                }
            ],
            "baseCommand": "java",
            "arguments": [
                {
                    "valueFrom": "${\n  return \"-jar\";\n}\n",
                    "position": 2
                },
                {
                    "valueFrom": "${\n  return \"/usr/local/bin/picard.jar\";\n}\n",
                    "position": 3
                },
                {
                    "valueFrom": "${\n  return \"MarkDuplicates\";\n}\n",
                    "position": 4
                }
            ],
            "doc": "USAGE: MarkDuplicates [options]\n\nDocumentation: http://broadinstitute.github.io/picard/command-line-overview.html#MarkDuplicates\n\nIdentifies duplicate reads.\nThis tool locates and tags duplicate reads in a BAM or SAM file, where duplicate reads are defined as originating\nfrom a single fragment of DNA.  Duplicates can arise during sample preparation e.g. library construction using PCR.\nSee also EstimateLibraryComplexity (https://broadinstitute.github.io/picard/command-line-overview.html#EstimateLibraryComplexity)\nfor additional notes on PCR duplication artifacts.  Duplicate reads can also result from a single amplification cluster,\nincorrectly detected as multiple clusters by the optical sensor of the sequencing instrument.\nThese duplication artifacts are referred to as optical duplicates.\n\nThe MarkDuplicates tool works by comparing sequences in the 5 prime positions of both reads and read-pairs in a\nSAM/BAM file.  An BARCODE_TAG option is available to facilitate duplicate marking using molecular barcodes.\nAfter duplicate reads are collected, the tool differentiates the primary and duplicate reads using an algorithm that\nranks reads by the sums of their base-quality scores (default method).\nThe tool's main output is a new SAM or BAM file, in which duplicates have been identified in the SAM flags field for\neach read.  Duplicates are marked with the hexadecimal value of 0x0400, which corresponds to a decimal value of 1024.\nIf you are not familiar with this type of annotation, please see the following blog post\n(https://www.broadinstitute.org/gatk/blog?id=7019) for additional information.\n\nAlthough the bitwise flag annotation indicates whether a read was marked as a duplicate, it does not identify the type\nof duplicate.  To do this, a new tag called the duplicate type (DT) tag was recently added as an optional output in\nthe 'optional field' section of a SAM/BAM file.  Invoking the TAGGING_POLICY option, you can instruct the program to\nmark all the duplicates (All), only the optical duplicates (OpticalOnly), or no duplicates (DontTag).\nThe records within the output of a SAM/BAM file will have values for the 'DT' tag (depending on the invoked\nTAGGING_POLICY), as either library/PCR-generated duplicates (LB), or sequencing-platform artifact duplicates (SQ).\nThis tool uses the READ_NAME_REGEX and the OPTICAL_DUPLICATE_PIXEL_DISTANCE options as the primary methods to\nidentify and differentiate duplicate types.  Set READ_NAME_REGEX to null to skip optical duplicate detection, e.g.\nfor RNA-seq or other data where duplicate sets are extremely large and estimating library complexity is not an aim.\nNote that without optical duplicate counts, library size estimation will be inaccurate.\n\nMarkDuplicates also produces a metrics file indicating the numbers of duplicates for both single- and paired-end reads.\nThe program can take either coordinate-sorted or query-sorted inputs, however the behavior is slightly different.\nWhen the input is coordinate-sorted, unmapped mates of mapped records and supplementary/secondary alignments are not\nmarked as duplicates.  However, when the input is query-sorted (actually query-grouped), then unmapped mates and\nsecondary/supplementary reads are not excluded from the duplication test and can be marked as duplicate reads.\nIf desired, duplicates can be removed using the REMOVE_DUPLICATE and REMOVE_SEQUENCING_DUPLICATES options.\n\nUsage example:\n\njava -jar picard.jar MarkDuplicates \\\n      I=input.bam \\\n      O=marked_duplicates.bam \\\n      M=marked_dup_metrics.txt\nPlease see MarkDuplicates (http://broadinstitute.github.io/picard/picard-metric-definitions.html#DuplicationMetrics) for detailed explanations of the output metrics.\n\nVersion: 2.8.3-SNAPSHOT\n\n\nOptions:\n\n--help\n-h                            Displays options specific to this tool.\n\n--stdhelp\n-H                            Displays options specific to this tool AND options common to all Picard command line\n                              tools.\n\n--version                     Displays program version.\n\nMAX_SEQUENCES_FOR_DISK_READ_ENDS_MAP=Integer\nMAX_SEQS=Integer              This option is obsolete. ReadEnds will always be spilled to disk.  Default value: 50000.\n                              This option can be set to 'null' to clear the default value.\n\nMAX_FILE_HANDLES_FOR_READ_ENDS_MAP=Integer\nMAX_FILE_HANDLES=Integer      Maximum number of file handles to keep open when spilling read ends to disk. Set this\n                              number a little lower than the per-process maximum number of file that may be open. This\n                              number can be found by executing the 'ulimit -n' command on a Unix system.  Default\n                              value: 8000. This option can be set to 'null' to clear the default value.\n\nSORTING_COLLECTION_SIZE_RATIO=Double\n                              This number, plus the maximum RAM available to the JVM, determine the memory footprint\n                              used by some of the sorting collections.  If you are running out of memory, try reducing\n                              this number.  Default value: 0.25. This option can be set to 'null' to clear the default\n                              value.\n\nBARCODE_TAG=String            Barcode SAM tag (ex. BC for 10X Genomics)  Default value: null.\n\nREAD_ONE_BARCODE_TAG=String   Read one barcode SAM tag (ex. BX for 10X Genomics)  Default value: null.\n\nREAD_TWO_BARCODE_TAG=String   Read two barcode SAM tag (ex. BX for 10X Genomics)  Default value: null.\n\nREMOVE_SEQUENCING_DUPLICATES=Boolean\n                              If true remove 'optical' duplicates and other duplicates that appear to have arisen from\n                              the sequencing process instead of the library preparation process, even if\n                              REMOVE_DUPLICATES is false. If REMOVE_DUPLICATES is true, all duplicates are removed and\n                              this option is ignored.  Default value: false. This option can be set to 'null' to clear\n                              the default value. Possible values: {true, false}\n\nTAGGING_POLICY=DuplicateTaggingPolicy\n                              Determines how duplicate types are recorded in the DT optional attribute.  Default value:\n                              DontTag. This option can be set to 'null' to clear the default value. Possible values:\n                              {DontTag, OpticalOnly, All}\n\nINPUT=String\nI=String                      One or more input SAM or BAM files to analyze. Must be coordinate sorted.  Default value:\n                              null. This option may be specified 0 or more times.\n\nOUTPUT=File\nO=File                        The output file to write marked records to  Required.\n\nMETRICS_FILE=File\nM=File                        File to write duplication metrics to  Required.\n\nREMOVE_DUPLICATES=Boolean     If true do not write duplicates to the output file instead of writing them with\n                              appropriate flags set.  Default value: false. This option can be set to 'null' to clear\n                              the default value. Possible values: {true, false}\n\nASSUME_SORTED=Boolean\nAS=Boolean                    If true, assume that the input file is coordinate sorted even if the header says\n                              otherwise. Deprecated, used ASSUME_SORT_ORDER=coordinate instead.  Default value: false.\n                              This option can be set to 'null' to clear the default value. Possible values: {true,\n                              false}  Cannot be used in conjuction with option(s) ASSUME_SORT_ORDER (ASO)\n\nASSUME_SORT_ORDER=SortOrder\nASO=SortOrder                 If not null, assume that the input file has this order even if the header says otherwise.\n                              Default value: null. Possible values: {unsorted, queryname, coordinate, duplicate}\n                              Cannot be used in conjuction with option(s) ASSUME_SORTED (AS)\n\nDUPLICATE_SCORING_STRATEGY=ScoringStrategy\nDS=ScoringStrategy            The scoring strategy for choosing the non-duplicate among candidates.  Default value:\n                              SUM_OF_BASE_QUALITIES. This option can be set to 'null' to clear the default value.\n                              Possible values: {SUM_OF_BASE_QUALITIES, TOTAL_MAPPED_REFERENCE_LENGTH, RANDOM}\n\nPROGRAM_RECORD_ID=String\nPG=String                     The program record ID for the @PG record(s) created by this program. Set to null to\n                              disable PG record creation.  This string may have a suffix appended to avoid collision\n                              with other program record IDs.  Default value: MarkDuplicates. This option can be set to\n                              'null' to clear the default value.\n\nPROGRAM_GROUP_VERSION=String\nPG_VERSION=String             Value of VN tag of PG record to be created. If not specified, the version will be\n                              detected automatically.  Default value: null.\n\nPROGRAM_GROUP_COMMAND_LINE=String\nPG_COMMAND=String             Value of CL tag of PG record to be created. If not supplied the command line will be\n                              detected automatically.  Default value: null.\n\nPROGRAM_GROUP_NAME=String\nPG_NAME=String                Value of PN tag of PG record to be created.  Default value: MarkDuplicates. This option\n                              can be set to 'null' to clear the default value.\n\nCOMMENT=String\nCO=String                     Comment(s) to include in the output file's header.  Default value: null. This option may\n                              be specified 0 or more times.\n\nREAD_NAME_REGEX=String        Regular expression that can be used to parse read names in the incoming SAM file. Read\n                              names are parsed to extract three variables: tile/region, x coordinate and y coordinate.\n                              These values are used to estimate the rate of optical duplication in order to give a more\n                              accurate estimated library size. Set this option to null to disable optical duplicate\n                              detection, e.g. for RNA-seq or other data where duplicate sets are extremely large and\n                              estimating library complexity is not an aim. Note that without optical duplicate counts,\n                              library size estimation will be inaccurate. The regular expression should contain three\n                              capture groups for the three variables, in order. It must match the entire read name.\n                              Note that if the default regex is specified, a regex match is not actually done, but\n                              instead the read name  is split on colon character. For 5 element names, the 3rd, 4th and\n                              5th elements are assumed to be tile, x and y values. For 7 element names (CASAVA 1.8),\n                              the 5th, 6th, and 7th elements are assumed to be tile, x and y values.  Default value:\n                              <optimized capture of last three ':' separated fields as numeric values>. This option can\n                              be set to 'null' to clear the default value.\n\nOPTICAL_DUPLICATE_PIXEL_DISTANCE=Integer\n                              The maximum offset between two duplicate clusters in order to consider them optical\n                              duplicates. The default is appropriate for unpatterned versions of the Illumina platform.\n                              For the patterned flowcell models, 2500 is moreappropriate. For other platforms and\n                              models, users should experiment to find what works best.  Default value: 100. This option\n                              can be set to 'null' to clear the default value.\n",
            "id": "#picard-markduplicates.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#picard-metadata.yaml",
                "http://schema.org/name": "picard",
                "http://schema.org/about": "Picard is a set of command line tools for manipulating high-throughput sequencing (HTS) data and formats such as SAM/BAM/CRAM and VCF. These file formats are defined in the Hts-specs repository. See especially the SAM specification and the VCF specification. Full list of supported tools:\n  AddCommentsToBam\n  AddOrReplaceReadGroups\n  BaitDesigner\n  BamToBfq\n  BamIndexStats\n  BedToIntervalList\n  BuildBamIndex\n  CalculateHsMetrics\n  CalculateReadGroupChecksum\n  CleanSam\n  CollectAlignmentSummaryMetrics\n  CollectBaseDistributionByCycle\n  CollectGcBiasMetrics\n  CollectHiSeqXPfFailMetrics\n  CollectHsMetrics\n  CollectIlluminaBasecallingMetrics\n  CollectIlluminaLaneMetrics\n  CollectInsertSizeMetrics\n  CollectJumpingLibraryMetrics\n  CollectMultipleMetrics\n  CollectOxoGMetrics\n  CollectQualityYieldMetrics\n  CollectRawWgsMetrics\n  CollectTargetedPcrMetrics\n  CollectRnaSeqMetrics\n  CollectRrbsMetrics\n  CollectSequencingArtifactMetrics\n  CollectVariantCallingMetrics\n  CollectWgsMetrics\n  CollectWgsMetricsFromQuerySorted\n  CollectWgsMetricsFromSampledSites\n  CollectWgsMetricsWithNonZeroCoverage\n  CompareMetrics\n  CompareSAMs\n  ConvertSequencingArtifactToOxoG\n  CreateSequenceDictionary\n  DownsampleSam\n  ExtractIlluminaBarcodes\n  EstimateLibraryComplexity\n  FastqToSam\n  FifoBuffer\n  FindMendelianViolations\n  FilterSamReads\n  FilterVcf\n  FixMateInformation\n  GatherBamFiles\n  GatherVcfs\n  GenotypeConcordance\n  IlluminaBasecallsToFastq\n  IlluminaBasecallsToSam\n  CheckIlluminaDirectory\n  CheckTerminatorBlock\n  IntervalListTools\n  LiftOverIntervalList\n  LiftoverVcf\n  MakeSitesOnlyVcf\n  MarkDuplicates\n  MarkDuplicatesWithMateCigar\n  MeanQualityByCycle\n  MergeBamAlignment\n  MergeSamFiles\n  MergeVcfs\n  NormalizeFasta\n  PositionBasedDownsampleSam\n  ExtractSequences\n  QualityScoreDistribution\n  RenameSampleInVcf\n  ReorderSam\n  ReplaceSamHeader\n  RevertSam\n  RevertOriginalBaseQualitiesAndAddMateCigar\n  SamFormatConverter\n  SamToFastq\n  ScatterIntervalsByNs\n  SetNmMdAndUqTags\n  SortSam\n  SortVcf\n  SplitSamByLibrary\n  UmiAwareMarkDuplicatesWithMateCigar\n  UpdateVcfSequenceDictionary\n  VcfFormatConverter\n  MarkIlluminaAdapters\n  SplitVcfs\n  ValidateSamFile\n  ViewSam\n  VcfToIntervalList\n",
                "http://schema.org/url": "https://broadinstitute.github.io/picard/",
                "http://schema.org/codeRepository": "https://github.com/broadinstitute/picard",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/MIT"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "2.9.0",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "Java",
                "http://schema.org/discussionUrl": [
                    "https://github.com/broadinstitute/picard/issues",
                    "http://gatkforums.broadinstitute.org/gatk"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Broad Institute"
                    }
                ]
            },
            "http://schema.org/downloadUrl": "https://github.com/SciDAP/workflows/blob/master/tools/picard-markduplicates.cwl",
            "http://schema.org/codeRepository": "https://github.com/SciDAP/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/author": {
                "class": "http://schema.org/Person",
                "http://schema.org/name": "Michael Kotliar",
                "http://schema.org/email": "mailto:michael.kotliar@cchmc.org",
                "http://schema.org/sameAs": [
                    {
                        "$import": "#0000-0002-6486-3898"
                    }
                ],
                "http://schema.org/worksFor": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Cincinnati Children's Hospital Medical Center",
                        "http://schema.org/location": "3333 Burnet Ave, Cincinnati, OH 45229-3026",
                        "http://schema.org/department": [
                            {
                                "class": "http://schema.org/Organization",
                                "http://schema.org/name": "Barski Lab"
                            }
                        ]
                    }
                ]
            }
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": "${\n  return  [\n            {\n              \"entry\": inputs.sort_input,\n              \"entryname\": inputs.sort_input.basename,\n              \"writable\": true\n            }\n          ]\n}\n"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var ext = function() { if (inputs.csi && !inputs.bai){ return '.csi'; } else { return '.bai'; } };",
                        "var default_bam = function() { if (inputs.trigger == true){ return inputs.sort_input.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+\".bam\"; } else { return inputs.sort_input.location.split('/').slice(-1)[0]; } };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/samtools:v1.4"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Generate BAI-format index for BAM files [default]. If input isn't cram.\n",
                    "id": "#samtools-sort-index.cwl/bai"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nif [ \"$0\" = \"true\" ]\nthen\n  echo \"Run: samtools index \" ${@:1}\n  samtools index \"${@:1}\"\nelse\n  echo \"Skip samtools index \" ${@:1}\nfi\n",
                    "inputBinding": {
                        "position": 20
                    },
                    "doc": "Bash function to run samtools index with all input parameters or skip it if trigger is false\n",
                    "id": "#samtools-sort-index.cwl/bash_script_index"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nif [ \"$0\" = \"true\" ]\nthen\n  echo \"Run: samtools sort \" ${@:1}\n  samtools sort \"${@:1}\"\nelse\n  echo \"Skip samtools sort \" ${@:1}\nfi\n",
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "Bash function to run samtools sort with all input parameters or skip it if trigger is false\n",
                    "id": "#samtools-sort-index.cwl/bash_script_sort"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Generate CSI-format index for BAM files. If input isn't cram.\n",
                    "id": "#samtools-sort-index.cwl/csi"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 24,
                        "prefix": "-m"
                    },
                    "doc": "Set minimum interval size for CSI indices to 2^INT [14]\n",
                    "id": "#samtools-sort-index.cwl/csi_interval"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-l"
                    },
                    "doc": "SORT: desired compression level for the final output file, ranging from 0 (uncompressed)\nor 1 (fastest but minimal compression) to 9 (best compression but slowest to write),\nsimilarly to gzip(1)'s compression level setting.\nIf -l is not used, the default compression level will apply.\n",
                    "id": "#samtools-sort-index.cwl/sort_compression_level"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 16
                    },
                    "doc": "Input only in.sam|in.bam|in.cram. Optionally could be supplemented with index file in secondaryFiles\n",
                    "id": "#samtools-sort-index.cwl/sort_input"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "-o",
                        "valueFrom": "${\n    if (self == \"\" || inputs.trigger == false){\n      return default_bam();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Write the final sorted output to FILE. Only out.bam|out.cram.\nIf output file extension is set to SAM, tool will fail on the index step\n",
                    "id": "#samtools-sort-index.cwl/sort_output_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "Set number of sorting and compression threads [1] (Only for sorting)\n",
                    "id": "#samtools-sort-index.cwl/threads"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "default": true,
                    "doc": "If true - run samtools, if false - return sort_input and optional index file in secondaryFiles, previously staged\ninto output directory.\n",
                    "id": "#samtools-sort-index.cwl/trigger"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.sort_output_filename == \"\" || inputs.trigger == false){\n      return default_bam();\n    } else {\n      return inputs.sort_output_filename;\n    }\n}\n"
                    },
                    "secondaryFiles": "${ if (inputs.trigger == true){ return self.basename + ext(); } else { return inputs.sort_input.secondaryFiles?inputs.sort_input.secondaryFiles:\"null\"; } }",
                    "id": "#samtools-sort-index.cwl/bam_bai_pair"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "arguments": [
                {
                    "valueFrom": "${ return inputs.trigger ? \"true\" : \"false\" }\n",
                    "position": 6
                },
                {
                    "valueFrom": "bam",
                    "position": 13,
                    "prefix": "-O"
                },
                {
                    "valueFrom": "$(inputs.threads?inputs.threads:1)",
                    "position": 15,
                    "prefix": "-@"
                },
                {
                    "valueFrom": ";",
                    "position": 17,
                    "shellQuote": false
                },
                {
                    "valueFrom": "bash",
                    "position": 18
                },
                {
                    "valueFrom": "-c",
                    "position": 19
                },
                {
                    "valueFrom": "${ return inputs.trigger ? \"true\" : \"false\" }\n",
                    "position": 21
                },
                {
                    "valueFrom": "$(inputs.bai?'-b':inputs.csi?'-c':[])",
                    "position": 23
                },
                {
                    "valueFrom": "$(inputs.threads?inputs.threads:1)",
                    "position": 25,
                    "prefix": "-@"
                },
                {
                    "valueFrom": "${\n    if (inputs.sort_output_filename == \"\" || inputs.trigger == false){\n      return default_bam();\n    } else {\n      return inputs.sort_output_filename;\n    }\n}\n",
                    "position": 26
                },
                {
                    "valueFrom": "${\n    if (inputs.sort_output_filename == \"\" || inputs.trigger == false){\n      return default_bam() + ext();\n    } else {\n      return inputs.sort_output_filename + ext();\n    }\n}\n",
                    "position": 27
                }
            ],
            "doc": "Tool to sort and index input BAM/SAM/CRAM.\nIf input `trigger` is set to `true` or isn't set at all (`true` is used by default), run `samtools sort` and\n`samtools index`, return sorted BAM and BAI/CSI index file.\nIf input `trigger` is set to `false`, return unchanged `sort_input` (BAM/SAM/CRAM) and index (BAI/CSI, if provided in\n`secondaryFiles`) files, previously staged into output directory.\n\nBefore execution `baseCommand`, `sort_input` and `secondaryFiles` (if provided) are staged into directory\nset as docker parameter `--workdir` (tool's output directory), using `InitialWorkDirRequirement`. Setting\n`writable: true` makes cwl-runner to make copies of the `sort_input` and `secondaryFiles` (if provided) and mount\nthem to docker container with `rw` mode as part of `--workdir` (if set to false, the files staged into output\ndirectory will be mounted to docker container separately with `ro` mode). Because both `samtools sort` and\n`samtools index` can overwrite files with the same names (and in case of `samtools sort` even the input file can be\noverwritten), we don't need to rename any of the staged files.\n\nTrigger logic is implemented in two bash scripts set by default as `bash_script_sort` and `bash_script_index` inputs.\nFor both of then, if the first argument $0 (which is `trigger` input) is true, run `samtools sort/index` with the rest\nof the arguments. If $0 is not true, skip `samtools sort/index` and return `sort_input` and `secondaryFiles`\n(if provided) staged into output directory.\n\nInput `trigger` is Boolean, but returns String, because of `valueFrom` field. The `valueFrom` is used, because if `trigger`\nis false, cwl-runner doesn't append this argument at all to the the `baseCommand` - new feature of CWL v1.0.2. Alternatively,\n`prefix` field could be used, but it causes changing in script logic.\n\nIf using `sort_output_filename`, the output file extension should be `*.bam`, because `samtools sort` defines the output\nfile format on the base of the file extension. If `*.sam` is sed as output filename, it cannot be usefully indexed\nby `samtools index`.\n\n`default_bam` function is used to generate output filename for `samtools sort` if input `sort_output_filename` is not\nset or when `trigger` is false and we need to return `sort_input` and `secondaryFiles` (if provided) files staged into\noutput directory. Output filename is generated on the base of `sort_input` basename with `.bam` extension by default.\n\n`ext` function is used to return the index file extension (BAI/CSI) based on `csi` and `bai` inputs according to the\nfollowing logic\n  `csi` &&  `bai`  => BAI\n !`csi` && !`bai ` => BAI\n  `csi` && !`bai ` => CSI\n",
            "id": "#samtools-sort-index.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#samtools-metadata.yaml",
                "http://schema.org/name": "samtools",
                "http://schema.org/about": "A suite of programs for interacting with high-throughput sequencing data. It consists of three separate repositories: Samtools (Reading/writing/editing/indexing/viewing SAM/BAM/CRAM format), BCFtools (Reading/writing BCF2/VCF/gVCF files and calling/filtering/summarising SNP and short indel sequence variants) and HTSlib (A C library for reading/writing high-throughput sequencing data).\n",
                "http://schema.org/url": "http://www.htslib.org/",
                "http://schema.org/codeRepository": "https://github.com/samtools/samtools.git",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/MIT",
                    "https://opensource.org/licenses/BSD-3-Clause"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "1.4",
                    "http://schema.org/applicationCategory": "commandline tool"
                },
                "http://schema.org/programmingLanguage": "C, Perl",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#btr509",
                        "http://schema.org/name": "(Li, 2011) A statistical framework for SNP calling, mutation discovery, association mapping and population genetical parameter estimation from sequencing data. Bioinformatics.",
                        "http://schema.org/url": "http://www.ncbi.nlm.nih.gov/pubmed/21903627"
                    },
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#btr076",
                        "http://schema.org/name": "(Li, 2011) Improving SNP discovery by base alignment quality. Bioinformatics.",
                        "http://schema.org/url": "http://www.ncbi.nlm.nih.gov/pubmed/21320865"
                    },
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#btp352",
                        "http://schema.org/name": "(Li et al., 2009) The Sequence Alignment/Map format and SAMtools. Bioinformatics.",
                        "http://schema.org/url": "http://www.ncbi.nlm.nih.gov/pubmed/19505943"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://lists.sourceforge.net/lists/listinfo/samtools-help",
                    "https://lists.sourceforge.net/lists/listinfo/samtools-devel"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Sanger Institute",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Heng Li",
                                "http://schema.org/description": "wrote most of the initial source codes of SAMtools and various converters."
                            }
                        ]
                    },
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Broad Institute",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Bob Handsaker",
                                "http://schema.org/description": "A major contributor to the\nSAM/BAM specification. He designed and implemented the BGZF format, the\nunderlying indexable compression format for the BAM format. BGZF does\nnot support arithmetic between file offsets.\n"
                            }
                        ]
                    },
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Beijing Genome Institute",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Jue Ruan",
                                "http://schema.org/description": "Designed and implemented the\nRAZF format, an alternative indexable compression format. RAZF is no longer\nused by or provided with SAMtools. Source code remains available in older\nSAMtools 0.1.x releases and from the standalone branch in the repository.\n"
                            }
                        ]
                    },
                    {
                        "class": "http://schema.org/Person",
                        "http://schema.org/name": "Colin Hercus",
                        "http://schema.org/description": "updated novo2sam.pl to support gapped alignment by novoalign."
                    },
                    {
                        "class": "http://schema.org/Person",
                        "http://schema.org/name": "Petr Danecek",
                        "http://schema.org/description": "contributed the header parsing library sam_header.c and sam2vcf.pl script."
                    }
                ]
            },
            "http://schema.org/name": "samtools-sort-index",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/samtools-sort-index.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage: samtools sort [options...] [in.bam]\nOptions:\n  -l INT     Set compression level, from 0 (uncompressed) to 9 (best)\n  -m INT     Set maximum memory per thread; suffix K/M/G recognized [768M]\n  -n         Sort by read name\n  -o FILE    Write final output to FILE rather than standard output\n  -T PREFIX  Write temporary files to PREFIX.nnnn.bam\n      --input-fmt-option OPT[=VAL]\n               Specify a single input file format option in the form\n               of OPTION or OPTION=VALUE\n  -O, --output-fmt FORMAT[,OPT[=VAL]]...\n               Specify output format (SAM, BAM, CRAM)\n      --output-fmt-option OPT[=VAL]\n               Specify a single output file format option in the form\n               of OPTION or OPTION=VALUE\n      --reference FILE\n               Reference sequence FASTA FILE [null]\n  -@, --threads INT\n               Number of additional threads to use [0]\n\nUsage: samtools index [-bc] [-m INT] <in.bam> [out.index]\nOptions:\n  -b       Generate BAI-format index for BAM files [default]\n  -c       Generate CSI-format index for BAM files\n  -m INT   Set minimum interval size for CSI indices to 2^INT [14]\n  -@ INT   Sets the number of threads [none]\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { let basename = inputs.bedgraph_file.location.split('/').slice(-1)[0]; let root = basename.split('.').slice(0,-1).join('.'); let ext = \".bigWig\"; return (root == \"\")?basename+ext:root+ext; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/ucscuserapps:v358"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 10
                    },
                    "doc": "Four column bedGraph file: <chrom> <start> <end> <value>\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/bedgraph_file"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "separate": false,
                        "position": 7,
                        "prefix": "-blockSize="
                    },
                    "doc": "Number of items to bundle in r-tree.  Default 256\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/block_size"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 11
                    },
                    "doc": "Two-column chromosome length file: <chromosome name> <size in bases>\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/chrom_length_file"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "separate": false,
                        "position": 6,
                        "prefix": "-itemsPerSlot="
                    },
                    "doc": "Number of data points bundled at lowest level. Default 1024\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/items_per_slot"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "If set, writes the output bigWig file to output_filename,\notherwise generates filename from default_output_filename()\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 5,
                        "prefix": "-unc"
                    },
                    "doc": "Disable compression\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/unc"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.output_filename == \"\"){\n      return default_output_filename();\n    } else {\n      return inputs.output_filename;\n    }\n}\n"
                    },
                    "id": "#ucsc-bedgraphtobigwig.cwl/bigwig_file"
                }
            ],
            "baseCommand": [
                "bedGraphToBigWig"
            ],
            "doc": "Tool converts bedGraph to bigWig file.\n\n`default_output_filename` function returns filename for generated bigWig if `output_filename` is not provided.\nDefault filename is generated on the base of `bedgraph_file` basename with the updated to `*.bigWig` extension.\n",
            "id": "#ucsc-bedgraphtobigwig.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#ucsc-metadata.yaml",
                "http://schema.org/name": "UCSC userApps",
                "http://schema.org/about": "UCSC genome browser bioinformatic utilities\n",
                "http://schema.org/url": "https://genome.ucsc.edu/util.html",
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "v358",
                    "http://schema.org/applicationCategory": "commandline tool",
                    "http://schema.org/downloadURL": "http://hgdownload.cse.ucsc.edu/admin/exe/userApps.v358.src.tgz"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ]
            },
            "http://schema.org/name": "ucsc-bedgraphtobigwig",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/ucsc-bedgraphtobigwig.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "usage:\n   bedGraphToBigWig in.bedGraph chrom.sizes out.bw\nwhere in.bedGraph is a four column file in the format:\n      <chrom> <start> <end> <value>\nand chrom.sizes is a two-column file/URL: <chromosome name> <size in bases>\nand out.bw is the output indexed big wig file.\nIf the assembly <db> is hosted by UCSC, chrom.sizes can be a URL like\n  http://hgdownload.cse.ucsc.edu/goldenPath/<db>/bigZips/<db>.chrom.sizes\nor you may use the script fetchChromSizes to download the chrom.sizes file.\nIf not hosted by UCSC, a chrom.sizes file can be generated by running\ntwoBitInfo on the assembly .2bit file.\nThe input bedGraph file must be sorted, use the unix sort command:\n  sort -k1,1 -k2,2n unsorted.bedGraph > sorted.bedGraph\noptions:\n   -blockSize=N - Number of items to bundle in r-tree.  Default 256\n   -itemsPerSlot=N - Number of data points bundled at lowest level. Default 1024\n   -unc - If set, do not use compression.\n"
        }
    ],
    "cwlVersion": "v1.0",
    "$schemas": [
        "http://schema.org/docs/schema_org_rdfa.html"
    ]
}
